@@ -16,7 +16,7 @@ projects = gl.projects.starred()
 projects = gl.projects.all()
 
 # Search projects
-projects = gl.projects.search('query')
+projects = gl.projects.list(search='keyword')
 # end list
 
 # get
@@ -31,7 +31,7 @@ project = gl.projects.create({'name': 'project1'})
 # end create
 
 # user create
-alice gl.users.list(username='alice')[0]
+alice = gl.users.list(username='alice')[0]
 user_project = gl.user_projects.create({'name': 'project',
                                         'user_id': alice.id})
 # end user create
