@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013-2015 Gauvain Pocentek <gauvain@pocentek.net>
+# Copyright (C) 2013-2017 Gauvain Pocentek <gauvain@pocentek.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -19,6 +19,7 @@
 from __future__ import print_function
 from __future__ import division
 from __future__ import absolute_import
+import importlib
 import inspect
 import itertools
 import json
@@ -31,10 +32,10 @@ import six
 import gitlab.config
 from gitlab.const import *  # noqa
 from gitlab.exceptions import *  # noqa
-from gitlab.objects import *  # noqa
+from gitlab.v3.objects import *  # noqa
 
 __title__ = 'python-gitlab'
-__version__ = '0.20'
+__version__ = '0.21'
 __author__ = 'Gauvain Pocentek'
 __email__ = 'gauvain@pocentek.net'
 __license__ = 'LGPL3'
@@ -65,18 +66,20 @@ class Gitlab(object):
         timeout (float): Timeout to use for requests to the GitLab server.
         http_username (str): Username for HTTP authentication
         http_password (str): Password for HTTP authentication
+        api_version (str): Gitlab API version to use (3 or 4)
     """
 
     def __init__(self, url, private_token=None, email=None, password=None,
                  ssl_verify=True, http_username=None, http_password=None,
-                 timeout=None):
+                 timeout=None, api_version='3'):
 
-        self._url = '%s/api/v3' % url
+        self._api_version = str(api_version)
+        self._url = '%s/api/v%s' % (url, api_version)
         #: Timeout to use for requests to gitlab server
         self.timeout = timeout
         #: Headers that will be used in request to GitLab
         self.headers = {}
-        self.set_token(private_token)
+        self._set_token(private_token)
         #: The user email
         self.email = email
         #: The user password (associated with email)
@@ -89,42 +92,52 @@ class Gitlab(object):
         #: Create a session object for requests
         self.session = requests.Session()
 
-        self.broadcastmessages = BroadcastMessageManager(self)
-        self.keys = KeyManager(self)
-        self.deploykeys = DeployKeyManager(self)
-        self.gitlabciymls = GitlabciymlManager(self)
-        self.gitignores = GitignoreManager(self)
-        self.groups = GroupManager(self)
-        self.hooks = HookManager(self)
-        self.issues = IssueManager(self)
-        self.licenses = LicenseManager(self)
-        self.namespaces = NamespaceManager(self)
-        self.notificationsettings = NotificationSettingsManager(self)
-        self.projects = ProjectManager(self)
-        self.runners = RunnerManager(self)
-        self.settings = ApplicationSettingsManager(self)
-        self.sidekiq = SidekiqManager(self)
-        self.snippets = SnippetManager(self)
-        self.users = UserManager(self)
-        self.teams = TeamManager(self)
-        self.todos = TodoManager(self)
+        objects = importlib.import_module('gitlab.v%s.objects' %
+                                          self._api_version)
+
+        self.broadcastmessages = objects.BroadcastMessageManager(self)
+        self.deploykeys = objects.DeployKeyManager(self)
+        self.gitlabciymls = objects.GitlabciymlManager(self)
+        self.gitignores = objects.GitignoreManager(self)
+        self.groups = objects.GroupManager(self)
+        self.hooks = objects.HookManager(self)
+        self.issues = objects.IssueManager(self)
+        self.licenses = objects.LicenseManager(self)
+        self.namespaces = objects.NamespaceManager(self)
+        self.notificationsettings = objects.NotificationSettingsManager(self)
+        self.projects = objects.ProjectManager(self)
+        self.runners = objects.RunnerManager(self)
+        self.settings = objects.ApplicationSettingsManager(self)
+        self.sidekiq = objects.SidekiqManager(self)
+        self.snippets = objects.SnippetManager(self)
+        self.users = objects.UserManager(self)
+        self.todos = objects.TodoManager(self)
+        if self._api_version == '3':
+            self.keys = objects.KeyManager(self)
+            self.teams = objects.TeamManager(self)
+        else:
+            self.dockerfiles = objects.DockerfileManager(self)
 
         # build the "submanagers"
-        for parent_cls in six.itervalues(globals()):
+        for parent_cls in six.itervalues(vars(objects)):
             if (not inspect.isclass(parent_cls)
-                or not issubclass(parent_cls, GitlabObject)
-                    or parent_cls == CurrentUser):
+                or not issubclass(parent_cls, objects.GitlabObject)
+                    or parent_cls == objects.CurrentUser):
                 continue
 
             if not parent_cls.managers:
                 continue
 
-            for var, cls, attrs in parent_cls.managers:
+            for var, cls_name, attrs in parent_cls.managers:
                 var_name = '%s_%s' % (self._cls_to_manager_prefix(parent_cls),
                                       var)
-                manager = cls(self)
+                manager = getattr(objects, cls_name)(self)
                 setattr(self, var_name, manager)
 
+    @property
+    def api_version(self):
+        return self._api_version
+
     def _cls_to_manager_prefix(self, cls):
         # Manage bad naming decisions
         camel_case = (cls.__name__
@@ -152,7 +165,8 @@ class Gitlab(object):
         return Gitlab(config.url, private_token=config.token,
                       ssl_verify=config.ssl_verify, timeout=config.timeout,
                       http_username=config.http_username,
-                      http_password=config.http_password)
+                      http_password=config.http_password,
+                      api_version=config.api_version)
 
     def auth(self):
         """Performs an authentication.
@@ -163,12 +177,17 @@ class Gitlab(object):
         success.
         """
         if self.private_token:
-            self.token_auth()
+            self._token_auth()
         else:
-            self.credentials_auth()
+            self._credentials_auth()
 
     def credentials_auth(self):
         """Performs an authentication using email/password."""
+        warnings.warn('credentials_auth() is deprecated and will be removed.',
+                      DeprecationWarning)
+        self._credentials_auth()
+
+    def _credentials_auth(self):
         if not self.email or not self.password:
             raise GitlabAuthenticationError("Missing email/password")
 
@@ -179,7 +198,16 @@ class Gitlab(object):
         """(gitlab.objects.CurrentUser): Object representing the user currently
             logged.
         """
-        self.set_token(self.user.private_token)
+        self._set_token(self.user.private_token)
+
+    def token_auth(self):
+        """Performs an authentication using the private token."""
+        warnings.warn('token_auth() is deprecated and will be removed.',
+                      DeprecationWarning)
+        self._token_auth()
+
+    def _token_auth(self):
+        self.user = CurrentUser(self)
 
     def version(self):
         """Returns the version and revision of the gitlab server.
@@ -202,17 +230,16 @@ class Gitlab(object):
 
         return self.version, self.revision
 
-    def token_auth(self):
-        """Performs an authentication using the private token."""
-        self.user = CurrentUser(self)
-
     def set_url(self, url):
         """Updates the GitLab URL.
 
         Args:
             url (str): Base URL of the GitLab server.
         """
-        self._url = '%s/api/v3' % url
+        warnings.warn('set_url() is deprecated, create a new Gitlab instance '
+                      'if you need an updated URL.',
+                      DeprecationWarning)
+        self._url = '%s/api/v%s' % (url, self._api_version)
 
     def _construct_url(self, id_, obj, parameters, action=None):
         if 'next_url' in parameters:
@@ -244,6 +271,12 @@ class Gitlab(object):
         Args:
             token (str): The private token.
         """
+        warnings.warn('set_token() is deprecated, use the private_token '
+                      'argument of the Gitlab constructor.',
+                      DeprecationWarning)
+        self._set_token(token)
+
+    def _set_token(self, token):
         self.private_token = token if token else None
         if token:
             self.headers["PRIVATE-TOKEN"] = token
@@ -257,6 +290,9 @@ class Gitlab(object):
             email (str): The user email or login.
             password (str): The user password.
         """
+        warnings.warn('set_credentials() is deprecated, use the email and '
+                      'password arguments of the Gitlab constructor.',
+                      DeprecationWarning)
         self.email = email
         self.password = password
 
@@ -483,7 +519,7 @@ class Gitlab(object):
 
         r = self._raw_delete(url, **params)
         raise_error_from_response(r, GitlabDeleteError,
-                                  expected_code=[200, 204])
+                                  expected_code=[200, 202, 204])
         return True
 
     def create(self, obj, **kwargs):
