@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013-2015 Gauvain Pocentek <gauvain@pocentek.net>
+# Copyright (C) 2013-2017 Gauvain Pocentek <gauvain@pocentek.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -83,10 +83,6 @@ class GitlabCancelError(GitlabOperationError):
     pass
 
 
-class GitlabBuildCancelError(GitlabCancelError):
-    pass
-
-
 class GitlabPipelineCancelError(GitlabCancelError):
     pass
 
@@ -95,6 +91,10 @@ class GitlabRetryError(GitlabOperationError):
     pass
 
 
+class GitlabBuildCancelError(GitlabCancelError):
+    pass
+
+
 class GitlabBuildRetryError(GitlabRetryError):
     pass
 
@@ -107,6 +107,22 @@ class GitlabBuildEraseError(GitlabRetryError):
     pass
 
 
+class GitlabJobCancelError(GitlabCancelError):
+    pass
+
+
+class GitlabJobRetryError(GitlabRetryError):
+    pass
+
+
+class GitlabJobPlayError(GitlabRetryError):
+    pass
+
+
+class GitlabJobEraseError(GitlabRetryError):
+    pass
+
+
 class GitlabPipelineRetryError(GitlabRetryError):
     pass
 
